/// File: main.dart
library jaguar.example.silly;

import 'dart:async';
import 'package:jaguar/jaguar.dart';
import 'package:jaguar/interceptors.dart';
import 'package:postgres/postgres.dart';

import 'package:jaguar_postgres/jaguar_postgres.dart';

part 'main.g.dart';

@Api(path: '/api')
class MongoDbExampleApi extends _$JaguarMongoDbExampleApi {
  /// NOTE: Example of using MongoDb interceptor in basic route
  @Get(path: '/post')
  // NOTE: This is how MongoDb interceptor is wrapped
  // around a route.
  // Now other interceptors and route handler methods can
  // request injection of opened connection using `@Input(MongoDb)`
  @WrapPostgresDb('localhost', 5432, 'postgres', username: 'postgres', password: 'dart_jaguar')
  @WrapEncodeMapToJson()
  Future<Map> mongoTest(
      // NOTE: This is how the opened mongodb connection is injected
      // into routes
      @Input(PostgresDb) PostgreSQLConnection db) async {
    await db.execute("delete FROM posts");
    await db.execute("insert into posts values (1, 'holla', 'jon')");
    List value = (await db.query("select * from posts WHERE _id = 1")).first;
    return {"Columns": value};
  }
}

Future<Null> main(List<String> args) async {
  MongoDbExampleApi api = new MongoDbExampleApi();

  Configuration configuration = new Configuration(multiThread: false);
  configuration.addApi(api);

  await serve(configuration);
}